""" Code is generated by ucloud-model, DO NOT EDIT IT. """

import typing


from ucloud.core.client import Client
from ucloud.services.umem.schemas import apis


class UMemClient(Client):
    def __init__(
        self, config: dict, transport=None, middleware=None, logger=None
    ):
        super(UMemClient, self).__init__(config, transport, middleware, logger)

    def create_umem_backup(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateUMemBackup - 创建分布式redis备份

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BackupName** (str) - (Required) 请求创建备份的名称 (范围[6-63],只能包含英文、数字以及符号-和_)
        - **SpaceId** (str) - (Required) 资源id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **BackupId** (str) - 备份Id

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateUMemBackupRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateUMemBackup", d, **kwargs)
        return apis.CreateUMemBackupResponseSchema().loads(resp)

    def create_umem_space(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateUMemSpace - 创建UMem内存空间

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 空间名称,长度(6<=size<=63)
        - **Size** (int) - (Required) 内存大小, 单位:GB, 范围[1~1024]
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year , Month, Dynamic, Trial 默认: Month
        - **CouponId** (str) - 使用的代金券id
        - **Protocol** (str) - 协议:memcache, redis (默认redis).注意:redis无single类型
        - **Quantity** (int) - 购买时长 默认: 1
        - **SubnetId** (str) -
        - **Type** (str) - 空间类型:single(无热备),double(热备)(默认: double)
        - **VPCId** (str) -

        **Response**

        - **SpaceId** (str) - 创建内存空间ID列表

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateUMemSpaceRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateUMemSpace", d, **kwargs)
        return apis.CreateUMemSpaceResponseSchema().loads(resp)

    def create_umem_cache_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateUMemcacheGroup - 创建单机Memcache

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 请求创建组的名称 范围[6-60]
        - **ChargeType** (str) - 计费模式，Year , Month, Dynamic 默认: Month
        - **ConfigId** (str) - 配置ID,目前仅支持默认配置id 默认配置id:"9a891891-c245-4b66-bce8-67e59430d67c"
        - **CouponId** (str) - 代金券ID
        - **Protocol** (str) -
        - **Quantity** (int) - 购买时长，默认为1
        - **Size** (int) - 每个节点的内存大小,单位GB,默认1GB 目前仅支持1/2/4/8/16/32这几档
        - **SubnetId** (str) -
        - **Tag** (str) - 业务组 默认：Default
        - **VPCId** (str) -
        - **Version** (str) - Memcache版本信息,默认为1.4.31
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **GroupId** (str) - 创建的组ID

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateUMemcacheGroupRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateUMemcacheGroup", d, **kwargs)
        return apis.CreateUMemcacheGroupResponseSchema().loads(resp)

    def create_uredis_backup(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateURedisBackup - 创建主备Redis备份

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BackupName** (str) - (Required) 请求创建组的名称 (范围[6-63],只能包含英文、数字以及符号-和_)
        - **GroupId** (str) - (Required) 资源id
        - **SlaveZone** (str) - 跨机房URedis，slave所在可用区（必须和Zone在同一Region，且不可相同）
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **BackupId** (str) - 备份id

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateURedisBackupRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateURedisBackup", d, **kwargs)
        return apis.CreateURedisBackupResponseSchema().loads(resp)

    def create_uredis_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateURedisGroup - 创建主备redis

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **HighAvailability** (str) - (Required) 是否开启高可用,enable或disable
        - **Name** (str) - (Required) 请求创建组的名称 (范围[6-63],只能包含英文、数字以及符号-和_)
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **AutoBackup** (str) - 是否自动备份,enable或disable，默认disable
        - **BackupId** (str) - 有此项代表从备份中创建，无代表正常创建
        - **BackupTime** (int) - 自动备份开始时间,范围[0-23],默认3点
        - **ChargeType** (str) - 计费模式，Year , Month, Dynamic 默认: Month
        - **ConfigId** (str) - 配置ID,目前支持 3.0版本配置ID:"03f58ca9-b64d-4bdd-abc7-c6b9a46fd801",3.2版本配置ID:"3e45ac48-f8a2-a9q2-261d-l342dab130gf", 4.0版本配置ID:"6c9298a3-9d7f-428c-b1d0-e87ab3b8a1ea",默认版本3.0,从备份创建为必传项
        - **CouponId** (str) - 代金券ID
        - **EnableIpV6** (bool) - 是否创建使用ipv6 资源， 默认为false， 或者不填， 创建ipv6为true
        - **MasterGroupId** (str) - Master Redis Group的ID，创建只读Slave时，必须填写
        - **Password** (str) - 初始化密码,需要 base64 编码
        - **Quantity** (int) - 购买时长，默认为1
        - **Size** (int) - 每个节点的内存大小,单位GB,默认1GB,目前仅支持1/2/4/8/16/32,六种
        - **SlaveZone** (str) - 跨机房URedis，slave所在可用区（必须和Zone在同一Region，且不可相同）
        - **SubnetId** (str) - 子网ID
        - **Tag** (str) - 业务组名称
        - **VPCId** (str) - VPC的ID
        - **Version** (str) - Redis版本信息(详见DescribeURedisVersion返回结果),默认版本3.0

        **Response**

        - **GroupId** (str) - 创建的组ID

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateURedisGroupRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateURedisGroup", d, **kwargs)
        return apis.CreateURedisGroupResponseSchema().loads(resp)

    def delete_umem_space(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DeleteUMemSpace - 删除UMem内存空间

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **SpaceId** (str) - (Required) UMem内存空间ID
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DeleteUMemSpaceRequestSchema().dumps(d)

        resp = self.invoke("DeleteUMemSpace", d, **kwargs)
        return apis.DeleteUMemSpaceResponseSchema().loads(resp)

    def delete_umem_cache_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DeleteUMemcacheGroup - 删除单机Memcache

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组ID
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DeleteUMemcacheGroupRequestSchema().dumps(d)

        resp = self.invoke("DeleteUMemcacheGroup", d, **kwargs)
        return apis.DeleteUMemcacheGroupResponseSchema().loads(resp)

    def delete_uredis_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DeleteURedisGroup - 删除主备redis

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组ID

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DeleteURedisGroupRequestSchema().dumps(d)

        resp = self.invoke("DeleteURedisGroup", d, **kwargs)
        return apis.DeleteURedisGroupResponseSchema().loads(resp)

    def describe_umem_backup(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemBackup - 查询分布式redis备份

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **SpaceId** (str) - (Required) 资源id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 分页显示的条目数, 默认值为10
        - **Offset** (int) - 分页显示的起始偏移, 默认值为0

        **Response**

        - **DataSet** (list) - 见 **UMemBackupSet** 模型定义

        **Response Model**

        **UMemBackupSet**

        - **BackupId** (str) - 空间的备份ID
        - **BackupName** (str) - 备份名称
        - **BackupType** (str) - 备份类型: auto(自动) ,manual(手动)
        - **BlockCount** (int) - 本次备份，分片的数量
        - **CreateTime** (int) - 创建时间
        - **State** (str) - Starting:备份中 Done:完成

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemBackupRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemBackup", d, **kwargs)
        return apis.DescribeUMemBackupResponseSchema().loads(resp)

    def describe_umem_backup_url(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemBackupURL - 获取分布式redis 备份下载链接

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BackupId** (str) - (Required) 备份Id
        - **SpaceId** (str) - (Required) 资源id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BlockId** (str) - 分片id

        **Response**

        - **BackupURL** (list) - 备份url，每个分片一个下载URL

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemBackupURLRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemBackupURL", d, **kwargs)
        return apis.DescribeUMemBackupURLResponseSchema().loads(resp)

    def describe_umem_block_info(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemBlockInfo - 拉取UDRedis分片信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - (Required) 分页显示的条目数, 默认值为10
        - **Offset** (int) - (Required) 分页显示的起始偏移, 默认值为0
        - **SpaceId** (str) - (Required) UMem内存资源ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **UMemBlockInfo** 模型定义

        **Response Model**

        **UMemBlockInfo**

        - **BlockId** (str) - 分片id
        - **BlockPort** (int) - 分片端口
        - **BlockSize** (int) - 容量单位GB
        - **BlockSlotBegin** (int) - 分片维护的键槽起始值
        - **BlockSlotEnd** (int) - 分片维护的键槽结束值
        - **BlockState** (str) - 实例状态 Starting // 创建中 Creating // 初始化中 CreateFail // 创建失败 Fail // 创建失败 Deleting // 删除中 DeleteFail // 删除失败 Running // 运行 Resizing // 容量调整中 ResizeFail // 容量调整失败 Configing // 配置中 ConfigFail // 配置失败Restarting // 重启中 SetPasswordFail //设置密码失败
        - **BlockUsedSize** (int) - 使用量单位MB
        - **BlockVip** (str) - 分片ip

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemBlockInfoRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemBlockInfo", d, **kwargs)
        return apis.DescribeUMemBlockInfoResponseSchema().loads(resp)

    def describe_umem_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemPrice - 获取UMem实例价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 购买umem大小,单位:GB,范围[1~1024]
        - **Type** (str) - (Required) 空间类型:single(无热备),double(热备)(默认: double)
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year， Month， Dynamic 如果不指定，则一次性获取三种计费
        - **Quantity** (int) - 购买UMem的时长，默认值为1

        **Response**

        - **DataSet** (list) - 见 **UMemPriceSet** 模型定义

        **Response Model**

        **UMemPriceSet**

        - **ChargeType** (str) - Year， Month， Dynamic，Trial
        - **ListPrice** (int) - 产品列表价
        - **OriginalPrice** (int) - 原价
        - **Price** (int) - 现价

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemPriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemPrice", d, **kwargs)
        return apis.DescribeUMemPriceResponseSchema().loads(resp)

    def describe_umem_space(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemSpace - 获取UMem内存空间列表

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 返回数据长度, 默认为20
        - **Offset** (int) - 数据偏移量, 默认为0
        - **Protocol** (str) - 协议类型: memcache, redis
        - **SpaceId** (str) - 内存空间ID (无ID，则获取所有)
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **UMemSpaceSet** 模型定义
        - **TotalCount** (int) - 根据过滤条件得到的总数

        **Response Model**

        **UMemSpaceAddressSet**

        - **IP** (str) - UMem实例访问IP
        - **Port** (int) - UMem实例访问Port

        **UMemSpaceSet**

        - **Address** (list) - 见 **UMemSpaceAddressSet** 模型定义
        - **ChargeType** (str) - Year, Month, Dynamic, Trial
        - **CreateTime** (int) - 创建时间
        - **ExpireTime** (int) - 到期时间
        - **Name** (str) - 内存空间名称
        - **Protocol** (str) - 协议类型: memcache, redis
        - **RewriteTime** (int) - 运维时间0   //0点1   //1点依次类推
        - **Size** (int) - 容量单位GB
        - **SpaceId** (str) - 内存空间ID
        - **State** (str) - Starting:创建中 Running:运行中 Fail:失败
        - **SubnetId** (str) -
        - **Tag** (str) -
        - **Type** (str) - 空间类型:single(无热备),double(热备)
        - **UsedSize** (int) - 使用量单位MB
        - **VPCId** (str) -
        - **Zone** (str) - 可用区，参见 `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemSpaceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemSpace", d, **kwargs)
        return apis.DescribeUMemSpaceResponseSchema().loads(resp)

    def describe_umem_upgrade_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemUpgradePrice - 获取UMem升级价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 购买UMem大小,单位:GB
        - **SpaceId** (str) - (Required) 需要升级的空间的SpaceId
        - **Type** (str) - (Required) 空间类型:single(无热备),double(热备)(默认: double)
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (dict) - 见 **PriceDataSet** 模型定义
        - **Price** (int) - 价格(兼容老版本)

        **Response Model**

        **PriceDataSet**

        - **CustomPrice** (int) - 用户折后价
        - **PurchaseValue** (int) - 资源有效期
        - **TotalPrice** (int) - 升降级资源的价格

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemUpgradePriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemUpgradePrice", d, **kwargs)
        return apis.DescribeUMemUpgradePriceResponseSchema().loads(resp)

    def describe_umem_cache_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemcacheGroup - 显示Memcache

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - 组的ID,如果指定则获取描述，否则为列表操 作,需指定Offset/Limit
        - **Limit** (int) - 分页显示的条目数, 默认值为20
        - **Offset** (int) - 分页显示的起始偏移, 默认值为0
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **UMemcacheGroupSet** 模型定义
        - **TotalCount** (int) - 组的总的节点个数

        **Response Model**

        **UMemcacheGroupSet**

        - **ChargeType** (str) - 计费类型:Year,Month,Dynamic 默认Dynamic
        - **ConfigId** (str) - 节点的配置ID
        - **CreateTime** (int) - 创建时间 (UNIX时间戳)
        - **ExpireTime** (int) - 过期时间 (UNIX时间戳)
        - **GroupId** (str) - 组ID
        - **ModifyTime** (int) - 修改时间 (UNIX时间戳)
        - **Name** (str) - 组名称
        - **Port** (int) - 节点分配的服务端口
        - **Size** (int) - 容量单位GB
        - **State** (str) - 状态标记 Creating // 初始化中 CreateFail // 创建失败 Deleting // 删除中 DeleteFail // 删除失败 Running // 运行 Resizing // 容量调整中 ResizeFail // 容量调整失败 Configing // 配置中 ConfigFail // 配置失败Restarting // 重启中
        - **SubnetId** (str) -
        - **Tag** (str) - 业务组名称
        - **UsedSize** (int) - 使用量单位MB
        - **VPCId** (str) -
        - **Version** (str) - Memcache版本信息,默认为1.4.31
        - **VirtualIP** (str) - 节点的虚拟IP地址

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemcacheGroupRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemcacheGroup", d, **kwargs)
        return apis.DescribeUMemcacheGroupResponseSchema().loads(resp)

    def describe_umem_cache_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemcachePrice - 获取umemcache组价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 容量大小,单位:GB 取值范围[1-32]
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - 计费模式，Year， Month， Dynamic，默认: Dynamic 默认: 获取所有计费模式的价格
        - **Quantity** (int) - 购买umemcache的时长，默认值为1
        - **Type** (str) - 空间类型:single(无热备),double(热备)(默认: double)

        **Response**

        - **DataSet** (list) - 见 **UMemcachePriceSet** 模型定义

        **Response Model**

        **UMemcachePriceSet**

        - **ChargeType** (str) - 计费模式，Year, Month, Dynamic
        - **ListPrice** (int) - 产品列表价
        - **OriginalPrice** (int) - 原价
        - **Price** (int) - 总价格

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeUMemcachePriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemcachePrice", d, **kwargs)
        return apis.DescribeUMemcachePriceResponseSchema().loads(resp)

    def describe_umem_cache_upgrade_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUMemcacheUpgradePrice - 获取umemcache升级价格信息

        **Request**

        - **GroupId** (str) - (Required) 需要升级的空间的GroupId,请参考DescribeUMemcacheGroup接口
        - **Size** (int) - (Required) 购买umemcache大小,单位:GB

        **Response**

        - **Price** (float) - 价格，单位：元

        """
        # build request
        d = {}
        req and d.update(req)
        d = apis.DescribeUMemcacheUpgradePriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUMemcacheUpgradePrice", d, **kwargs)
        return apis.DescribeUMemcacheUpgradePriceResponseSchema().loads(resp)

    def describe_uredis_backup(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisBackup - 查询主备redis备份

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - 组的ID
        - **Limit** (int) - 分页显示的条目数, 默认值为10
        - **Offset** (int) - 分页显示的起始偏移, 默认值为0

        **Response**

        - **DataSet** (list) - 见 **URedisBackupSet** 模型定义
        - **TotalCount** (int) - 用户名下总的备份个数

        **Response Model**

        **URedisBackupSet**

        - **BackupId** (str) - 备份ID
        - **BackupName** (str) - 备份的名称
        - **BackupSize** (int) - 备份文件大小, 以字节为单位
        - **BackupTime** (int) - 备份时间 (UNIX时间戳)
        - **BackupType** (str) - 备份类型: Manual 手动 Auto 自动
        - **GroupId** (str) - 对应的实例ID
        - **GroupName** (str) - 组名称
        - **State** (str) - 备份的状态: Backuping 备份中 Success 备份成功 Error 备份失败 Expired 备份过期
        - **Zone** (str) - 可用区，参见 `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisBackupRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisBackup", d, **kwargs)
        return apis.DescribeURedisBackupResponseSchema().loads(resp)

    def describe_uredis_backup_url(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisBackupURL - 获取主备Redis备份下载链接

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BackupId** (str) - (Required) 备份ID
        - **GroupId** (str) - 实例名称
        - **RegionFlag** (bool) - 是否是跨机房URedis(默认false)
        - **SlaveZone** (str) - 跨机房URedis，slave所在可用区（必须和Zone在同一Region，且不可相同）
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **BackupPath** (str) - 备份文件公网的地址
        - **BackupURL** (str) - 备份文件公网的地址

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisBackupURLRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisBackupURL", d, **kwargs)
        return apis.DescribeURedisBackupURLResponseSchema().loads(resp)

    def describe_uredis_config(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisConfig - 查询主备Redis所有配置文件

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **RegionFlag** (bool) - (Required) 是否是跨机房URedis(默认false)
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ConfigId** (str) - 配置文件ID
        - **Limit** (int) - 页显示的条目数, 默认值为10
        - **Offset** (int) - 页显示的起始偏移, 默认值为0
        - **Version** (str) - Redis版本号

        **Response**

        - **DataSet** (list) - 见 **URedisConfigSet** 模型定义
        - **TotalCount** (int) - 根据过滤条件得到的总数

        **Response Model**

        **URedisConfigSet**

        - **ConfigId** (str) - 配置ID
        - **CreateTime** (int) - 创建时间 (UNIX时间戳)
        - **Description** (str) - 配置描述
        - **IsModify** (str) - 置是否可以修改
        - **ModifyTime** (int) - 修改时间 (UNIX时间戳)
        - **Name** (str) - 配置名称
        - **RegionFlag** (bool) - 是否是跨机房URedis(默认false)
        - **State** (str) - 配置所处的状态
        - **Version** (str) - 配置对应的Redis版本
        - **Zone** (str) - Zone

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisConfigRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisConfig", d, **kwargs)
        return apis.DescribeURedisConfigResponseSchema().loads(resp)

    def describe_uredis_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisGroup - 查询主备Redis

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - 组的ID,如果指定则获取描述，否则为列表操 作,需指定Offset/Limit
        - **Limit** (int) - 分页显示的条目数, 默认值为20
        - **Offset** (int) - 分页显示的起始偏移, 默认值为0
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **URedisGroupSet** 模型定义
        - **TotalCount** (int) - 组的总的节点个数

        **Response Model**

        **URedisGroupSet**

        - **AutoBackup** (str) - 是否需要自动备份,enable,disable
        - **BackupTime** (int) - 组自动备份开始时间,单位小时计,范围[0-23]
        - **ChargeType** (str) - 计费类型:Year,Month,Dynamic 默认Dynamic
        - **ConfigId** (str) - 节点的配置ID
        - **CreateTime** (int) - 创建时间 (UNIX时间戳)
        - **ExpireTime** (int) - 过期时间 (UNIX时间戳)
        - **GroupId** (str) - 组ID
        - **GroupName** (str) - 组名称
        - **HighAvailability** (str) - 是否开启高可用,enable,disable
        - **MemorySize** (int) - 容量单位GB
        - **ModifyTime** (int) - 修改时间 (UNIX时间戳)
        - **Name** (str) - 组名称
        - **Port** (int) - 节点分配的服务端口
        - **Protocol** (str) - 协议
        - **RewriteTime** (int) - 返回运维时间 0 //0点 1 //1点 以此类推
        - **Role** (str) - 实例类型
        - **Size** (int) - 容量单位GB
        - **SlaveZone** (str) - 跨机房URedis，slave redis所在可用区，参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **State** (str) - 状态标记 Creating // 初始化中 CreateFail // 创建失败 Deleting // 删除中 DeleteFail // 删除失败 Running // 运行 Resizing // 容量调整中 ResizeFail // 容量调整失败 Configing // 配置中 ConfigFail // 配置失败
        - **SubnetId** (str) - 子网
        - **Tag** (str) - 业务组名称
        - **Type** (str) - 空间类型:single(无热备),double(热备)
        - **UsedSize** (int) - 使用量单位MB
        - **VPCId** (str) - VPCId
        - **Version** (str) - Redis版本信息
        - **VirtualIP** (str) - 节点的虚拟IP地址
        - **Zone** (str) - 实例所在可用区，或者master redis所在可用区，参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisGroupRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisGroup", d, **kwargs)
        return apis.DescribeURedisGroupResponseSchema().loads(resp)

    def describe_uredis_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisPrice - 取uredis价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 量大小,单位:GB  取值范围[1-32]
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - 计费模式，Year， Month， Dynamic；如果不指定，则一次性获取三种计费
        - **ProductType** (str) - 产品类型：MS_Redis（标准主备版），S_Redis（从库），默认为MS_Redis
        - **Quantity** (int) - 计费模式为Dynamic时，购买的时长, 默认为1
        - **RegionFlag** (bool) - 是否是跨机房URedis(默认false)

        **Response**

        - **DataSet** (list) - 见 **URedisPriceSet** 模型定义

        **Response Model**

        **URedisPriceSet**

        - **ChargeType** (str) - Year， Month， Dynamic，Trial
        - **ListPrice** (int) - 产品列表价
        - **OriginalPrice** (int) - 原价
        - **Price** (int) - 总价格

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisPriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisPrice", d, **kwargs)
        return apis.DescribeURedisPriceResponseSchema().loads(resp)

    def describe_uredis_slowlog(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisSlowlog - 查询URedis慢日志

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 资源ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 分页显示的条目数，默认为10

        **Response**

        - **DataSet** (list) - 见 **URedisSlowlogSet** 模型定义
        - **TotalCount** (int) - 总条目数

        **Response Model**

        **URedisSlowlogSet**

        - **Command** (str) - 查询命令
        - **SpendTime** (int) - 查询消耗的时间
        - **StartTime** (int) - 查询发生的时间

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisSlowlogRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisSlowlog", d, **kwargs)
        return apis.DescribeURedisSlowlogResponseSchema().loads(resp)

    def describe_uredis_upgrade_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisUpgradePrice - 获取uredis升级价格信息

        **Request**

        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 要升级的空间的GroupId,请参考DescribeURedisGroup接口
        - **Size** (int) - (Required) 购买uredis大小,单位:GB,范围是[1-32]
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **Price** (float) - 扩容差价，单位: 元，保留小数点后两位有效数字

        """
        # build request
        d = {
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisUpgradePriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisUpgradePrice", d, **kwargs)
        return apis.DescribeURedisUpgradePriceResponseSchema().loads(resp)

    def describe_uredis_version(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeURedisVersion - 获取主Redis可用版本

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **URedisVersionSet** 模型定义
        - **TotalCount** (int) - 总版本个数

        **Response Model**

        **URedisVersionSet**

        - **Version** (str) - Redis版本

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeURedisVersionRequestSchema().dumps(d)

        resp = self.invoke("DescribeURedisVersion", d, **kwargs)
        return apis.DescribeURedisVersionResponseSchema().loads(resp)

    def get_umem_space_state(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """GetUMemSpaceState - 获取UMem内存空间列表

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **SpaceId** (str) - (Required) 内存空间ID
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **State** (str) - Starting:创建中 Running:运行中 Fail:失败

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.GetUMemSpaceStateRequestSchema().dumps(d)

        resp = self.invoke("GetUMemSpaceState", d, **kwargs)
        return apis.GetUMemSpaceStateResponseSchema().loads(resp)

    def modify_umem_space_name(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ModifyUMemSpaceName - 修改UMem内存空间名称

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 新的名称,长度(6<=size<=63)
        - **SpaceId** (str) - (Required) UMem内存空间ID
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ModifyUMemSpaceNameRequestSchema().dumps(d)

        resp = self.invoke("ModifyUMemSpaceName", d, **kwargs)
        return apis.ModifyUMemSpaceNameResponseSchema().loads(resp)

    def modify_uredis_group_name(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ModifyURedisGroupName - 修改主备redis名称

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组的ID
        - **Name** (str) - (Required) Redis组名称 (范围[6-63],只能包含英文、数字以及符号-和_)

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ModifyURedisGroupNameRequestSchema().dumps(d)

        resp = self.invoke("ModifyURedisGroupName", d, **kwargs)
        return apis.ModifyURedisGroupNameResponseSchema().loads(resp)

    def modify_uredis_group_password(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ModifyURedisGroupPassword - 修改主备密码/重置密码

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组的ID
        - **Password** (str) - (Required) 新密码字符串，要求长度为6~36个字符,且只能包含英文、数字以及-和下划线；并且需要base64加密；如要取消密码，此值为空字符串，
        - **ResourceType** (str) -
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ModifyURedisGroupPasswordRequestSchema().dumps(d)

        resp = self.invoke("ModifyURedisGroupPassword", d, **kwargs)
        return apis.ModifyURedisGroupPasswordResponseSchema().loads(resp)

    def resize_umem_space(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ResizeUMemSpace - 调整内存空间容量

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 内存大小, 单位:GB (需要大于原size,<= 1024)
        - **SpaceId** (str) - (Required) UMem 内存空间Id
        - **CouponId** (str) - 使用的代金券Id
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ResizeUMemSpaceRequestSchema().dumps(d)

        resp = self.invoke("ResizeUMemSpace", d, **kwargs)
        return apis.ResizeUMemSpaceResponseSchema().loads(resp)

    def resize_uredis_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ResizeURedisGroup - 调整主备redis容量

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组ID
        - **Size** (int) - (Required) 内存大小, 单位:GB (需要大于原size,且小于等于32) 目前仅支持1/2/4/8/16/32 G 六种容量规格
        - **CouponId** (int) - 代金券ID 请参考DescribeCoupon接口

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ResizeURedisGroupRequestSchema().dumps(d)

        resp = self.invoke("ResizeURedisGroup", d, **kwargs)
        return apis.ResizeURedisGroupResponseSchema().loads(resp)

    def restart_umem_cache_group(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """RestartUMemcacheGroup - 重启单机Memcache

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **GroupId** (str) - (Required) 组的ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.RestartUMemcacheGroupRequestSchema().dumps(d)

        resp = self.invoke("RestartUMemcacheGroup", d, **kwargs)
        return apis.RestartUMemcacheGroupResponseSchema().loads(resp)

    def update_uredis_backup_strategy(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """UpdateURedisBackupStrategy - URedisBackupStrategy

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **BackupTime** (str) - (Required) 备份时间，默认为0
        - **GroupId** (str) - (Required) 组的ID
        - **AutoBackup** (str) - 是否打开默认备份功能。enable(打开)，disable(关闭)，默认enable
        - **SlaveZone** (str) - 跨机房URedis，slave所在可用区（必须和Zone在同一Region，且不可相同）
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.UpdateURedisBackupStrategyRequestSchema().dumps(d)

        resp = self.invoke("UpdateURedisBackupStrategy", d, **kwargs)
        return apis.UpdateURedisBackupStrategyResponseSchema().loads(resp)



import random
import base64
import typing

_lowercase = "abcdefghijklmnopqrstuvwxyz"
_uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
_numbers = "0123456789"
_specials = "_"


def gen_password(
    n: int,
    lower_letters: str = _lowercase,
    upper_letters: str = _uppercase,
    number_letters: str = _numbers,
    special_letters: str = _specials,
    min_lower: int = 1,
    min_upper: int = 1,
    min_number: int = 1,
    min_specials: int = 1,
):
    """generate password for any resource

    >>> len(gen_password(20))
    20

    :param int n: password total length
    :param str lower_letters: all lowercase letters
    :param str upper_letters: all uppercase letters
    :param str number_letters: all number letters
    :param str special_letters: all special letters
    :param int min_lower: minimal number of lowercase letters
    :param int min_upper: minimal number of uppercase letters
    :param int min_number: minimal number of number letters
    :param int min_specials: minimal number of special letters
    :return:
    """
    all_letters = "".join(
        [lower_letters, upper_letters, number_letters, special_letters]
    )
    minimal_total = min_lower + min_upper + min_number + min_specials
    if n < minimal_total:
        raise ValueError(
            (
                "the length of password must be larger than "
                "total minimal letters number"
            )
        )

    minimal_letters = "".join(
        [
            gen_string(lower_letters, min_lower),
            gen_string(upper_letters, min_upper),
            gen_string(number_letters, min_number),
            gen_string(special_letters, min_specials),
        ]
    )

    additional_letters = random.sample(all_letters, n - minimal_total)
    results = list(minimal_letters) + additional_letters
    random.shuffle(results)
    return "".join(results)


def gen_string(letters: str, length: int):
    return "".join([random.choice(letters) for i in range(length)])


def first(arr: typing.List[typing.Any]) -> typing.Any:
    if len(arr) == 0:
        return None
    return arr[0]


def b64encode(s: str) -> str:
    """base64 encode

    :param str s: input string
    :return: base64 string
    """
    return base64.b64encode(s.encode()).decode()


def b64decode(s: str) -> str:
    """base64 decode

    :param str s: base64 string
    :return: output string
    """
    return base64.b64decode(s.encode()).decode()



import functools
import logging

logger = logging.getLogger("ucloud")


def deprecated(instead_of="", message=""):
    """deprecated is a decorator to mark a function is deprecated.
    it will logging warning when this function called

    >>> @deprecated(instead_of="new_function")
    ... def an_old_function():
    ...     pass
    """

    def deco(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            msg = [
                "this function/method `{}` is deprecated".format(fn.__name__)
            ]
            instead_of and msg.append(
                "please use `{}` instead".format(instead_of)
            )
            message and msg.append(message)
            logger.warning(", ".join(msg))
            return fn(*args, **kwargs)

        return wrapper

    return deco



import typing
import logging
import sys

from ucloud import version
from ucloud.core.client._cfg import Config
from ucloud.core.transport import (
    Transport,
    RequestsTransport,
    Request,
    SSLOption,
    http,
)
from ucloud.core.typesystem import encoder
from ucloud.core.utils import log
from ucloud.core.utils.middleware import Middleware
from ucloud.core import auth, exc


class Client:
    def __init__(
        self,
        config: dict,
        transport: typing.Optional[Transport] = None,
        middleware: typing.Optional[Middleware] = None,
        logger: typing.Optional[logging.Logger] = None,
    ):
        cfg, cred = self._parse_dict_config(config)
        self.config = cfg
        self.credential = cred
        self.transport = transport or RequestsTransport()
        self.logger = logger or log.default_logger
        if middleware is None:
            middleware = Middleware()
            middleware.response(self.logged_response_handler)
            middleware.request(self.logged_request_handler)
            middleware.exception(self.logged_exception_handler)
        self._middleware = middleware

    def invoke(self, action: str, args: dict = None, **options) -> dict:
        """invoke will invoke the action with arguments data and options

        :param str action: the api action, like `CreateUHostInstance`
        :param dict args: arguments of api(action), see doc: `UCloud API Documentation <https://docs.ucloud.cn/api>`__
        :return:
        """
        retries = 0
        max_retries = options.get("max_retries") or self.config.max_retries
        timeout = options.get("timeout") or self.config.timeout

        while retries <= max_retries:
            try:
                return self._send(
                    action, args or {}, max_retries=max_retries, timeout=timeout
                )
            except exc.UCloudException as e:
                if e.retryable and retries != max_retries:
                    logging.info(
                        "Retrying {action}: {args}".format(
                            action=action, args=args
                        )
                    )
                    retries += 1
                    continue
                raise e

    @property
    def middleware(self) -> Middleware:
        return self._middleware

    def logged_request_handler(self, req):
        action = req.get("Action", "")
        self.logger.info("[request] {} {}".format(action, req))
        return req

    def logged_response_handler(self, resp, http_resp: http.Response = None):
        action = resp.get("Action", "")
        request_uuid = http_resp and http_resp.request_uuid
        self.logger.info(
            "[response] [{}] {} {}".format(request_uuid or "*", action, resp)
        )
        return resp

    def logged_exception_handler(self, e: Exception):
        if isinstance(e, exc.RetCodeException):
            self.logger.warning(e)
        else:
            self.logger.exception(e)
        return e

    @staticmethod
    def _parse_dict_config(
        config: dict,
    ) -> typing.Tuple[Config, auth.Credential]:
        return Config.from_dict(config), auth.Credential.from_dict(config)

    def _send(self, action: str, args: dict, max_retries, timeout) -> dict:
        args["Action"] = action

        # inject request middleware
        for handler in self.middleware.request_handlers:
            args = handler(args)

        # send http request
        try:
            req = self._build_http_request(args)

            resp = self.transport.send(
                req,
                ssl_option=SSLOption(
                    self.config.ssl_verify,
                    self.config.ssl_cacert,
                    self.config.ssl_cert,
                    self.config.ssl_key,
                ),
                timeout=timeout,
                max_retries=max_retries,
            )
            data = resp.json()
        except Exception as e:
            for handler in self.middleware.exception_handlers:
                handler(e)
            raise e

        # inject response middleware
        for handler in self.middleware.response_handlers:
            data = handler(data, resp)

        # return when successful
        if int(data.get("RetCode", -1)) == 0:
            return data

        # inject exception middleware
        ret_code_exc = exc.RetCodeException(
            action=req.data.get("Action", ""),
            code=int(data.get("RetCode", 0)),
            message=data.get("Message", ""),
            request_uuid=resp.request_uuid,
        )
        for handler in self.middleware.exception_handlers:
            handler(ret_code_exc)
        raise ret_code_exc

    def _build_http_request(self, args: dict) -> Request:
        config = {
            "Region": self.config.region,
            "ProjectId": self.config.project_id,
        }
        payload = {k: v for k, v in config.items() if v is not None}
        payload.update({k: v for k, v in args.items() if v is not None})
        payload = encoder.encode(payload)
        payload["Signature"] = self.credential.verify_ac(payload)

        return Request(
            url=self.config.base_url,
            method="post",
            data=payload,
            headers={
                "User-Agent": self._build_user_agent(),
                "Content-Type": "application/x-www-form-urlencoded",
            },
        )

    def _build_user_agent(self) -> str:
        python_version = "{v[0]}.{v[1]}.{v[2]}".format(v=sys.version_info)
        user_agent = "Python/{python_version} Python-SDK/{sdk_version}".format(
            python_version=python_version, sdk_version=version.version
        ) + (self.config.user_agent or "")
        return user_agent

    def __repr__(self):
        return '<{}(region="{}")>'.format(
            self.__class__.__name__, self.config.region
        )



from ucloud.testing.driver import _scenario


class Specification:
    def __init__(self):
        self.scenarios = []

    def scenario(self, id_, title="", owners=None):
        scenario = _scenario.Scenario(self, id_, title, owners)
        self.scenarios.append(scenario)
        return scenario

    @property
    def status(self):
        if all([item.status == "skipped" for item in self.scenarios]):
            status = "skipped"
        elif any([item.status == "failed" for item in self.scenarios]):
            status = "failed"
        else:
            status = "passed"
        return status

    @property
    def start_time(self):
        times = [
            item.start_time
            for item in self.scenarios
            if item.status != "skipped"
        ]
        return min(times) if times else 0

    @property
    def end_time(self):
        times = [
            item.end_time for item in self.scenarios if item.status != "skipped"
        ]
        return max(times) if times else 0

    def json(self):
        return {
            "status": self.status,
            "execution": {
                "duration": self.end_time - self.start_time,
                "start_time": self.start_time,
                "end_time": self.end_time,
            },
            "scenarios": [item.json() for item in self.scenarios],
            "passed_count": len(
                [1 for item in self.scenarios if item.status == "passed"]
            ),
            "failed_count": len(
                [1 for item in self.scenarios if item.status == "failed"]
            ),
            "skipped_count": len(
                [1 for item in self.scenarios if item.status == "skipped"]
            ),
        }



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

import typing


from ucloud.core.client import Client
from ucloud.services.ipsecvpn.schemas import apis


class IPSecVPNClient(Client):
    def __init__(
        self, config: dict, transport=None, middleware=None, logger=None
    ):
        super(IPSecVPNClient, self).__init__(
            config, transport, middleware, logger
        )

    def describe_remote_vpn_gateway(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeRemoteVPNGateway - 获取客户VPN网关信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 数据分页值, 默认为20
        - **Offset** (int) - 数据偏移量, 默认为0
        - **RemoteVPNGatewayIds** (list) - 客户VPN网关的资源ID，例如RemoteVPNGatewayIds.0代表希望获取客户VPN网关1的信息，RemoteVPNGatewayIds.1代表客户VPN网关2，如果为空，则返回当前Region中所有客户VPN网关实例的信息
        - **Tag** (str) - 业务组名称，若指定则返回业务组下所有客户VPN网关信息

        **Response**

        - **DataSet** (list) - 见 **RemoteVPNGatewayDataSet** 模型定义
        - **TotalCount** (int) - 符合条件的客户VPN网关总数

        **Response Model**

        **RemoteVPNGatewayDataSet**

        - **ActiveTunnels** (str) - 活跃的隧道id
        - **CreateTime** (int) - 创建时间
        - **Remark** (str) - 备注
        - **RemoteVPNGatewayAddr** (str) - 客户网关IP地址
        - **RemoteVPNGatewayId** (str) - 客户网关ID
        - **RemoteVPNGatewayName** (str) - 客户网关名称
        - **Tag** (str) - 用户组
        - **TunnelCount** (int) - 活跃的隧道数量

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeRemoteVPNGatewayRequestSchema().dumps(d)

        resp = self.invoke("DescribeRemoteVPNGateway", d, **kwargs)
        return apis.DescribeRemoteVPNGatewayResponseSchema().loads(resp)

    def describe_vpn_tunnel(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeVPNTunnel - 获取VPN隧道信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 数据分页值, 默认为20
        - **Offset** (int) - 数据偏移量, 默认为0
        - **Tag** (str) - 业务组名称，若指定则返回指定的业务组下的所有VPN网关的信息
        - **VPNTunnelIds** (list) - VPN隧道的资源ID，例如VPNTunnelIds.0代表希望获取信息的VPN隧道1，VPNTunneIds.1代表VPN隧道2，如果为空，则返回当前Region中所有的VPN隧道实例

        **Response**

        - **DataSet** (list) - 见 **VPNTunnelDataSet** 模型定义
        - **TotalCount** (int) - VPN隧道总数

        **Response Model**

        **IPSecData**

        - **IPSecAuthenticationAlgorithm** (str) - IPSec通道中使用的认证算法
        - **IPSecEncryptionAlgorithm** (str) - IPSec通道中使用的加密算法
        - **IPSecLocalSubnetIds** (list) - 指定VPN连接的本地子网，用逗号分隔
        - **IPSecPFSDhGroup** (str) - 是否开启PFS功能,Disable表示关闭，数字表示DH组
        - **IPSecProtocol** (str) - 使用的安全协议，ESP或AH
        - **IPSecRemoteSubnets** (list) - 指定VPN连接的客户网段，用逗号分隔
        - **IPSecSALifetime** (str) - IPSec中SA的生存时间
        - **IPSecSALifetimeBytes** (str) - IPSec中SA的生存时间（以字节计）

        **IKEData**

        - **IKEAuthenticationAlgorithm** (str) - IKE认证算法
        - **IKEDhGroup** (str) - IKEDH组
        - **IKEEncryptionAlgorithm** (str) - IKE加密算法
        - **IKEExchangeMode** (str) - IKEv1协商模式
        - **IKELocalId** (str) - IKE本地ID标识
        - **IKEPreSharedKey** (str) - IKE预共享秘钥
        - **IKERemoteId** (str) - IKE对端ID标识
        - **IKESALifetime** (str) - IKE秘钥生存时间
        - **IKEVersion** (str) - IKE版本

        **VPNTunnelDataSet**

        - **CreateTime** (int) - 创建时间
        - **IKEData** (dict) - 见 **IKEData** 模型定义
        - **IPSecData** (dict) - 见 **IPSecData** 模型定义
        - **Remark** (str) - 备注
        - **RemoteVPNGatewayId** (str) - 对端网关Id
        - **RemoteVPNGatewayName** (str) - 对端网关名字
        - **Tag** (str) - 用户组
        - **VPCId** (str) - 所属VPCId
        - **VPCName** (str) - 所属VOC名字
        - **VPNGatewayId** (str) - 所属VPN网关id
        - **VPNGatewayName** (str) - VPN网关名字
        - **VPNTunnelId** (str) - 隧道id
        - **VPNTunnelName** (str) - 隧道名称

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DescribeVPNTunnelRequestSchema().dumps(d)

        resp = self.invoke("DescribeVPNTunnel", d, **kwargs)
        return apis.DescribeVPNTunnelResponseSchema().loads(resp)



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

import typing


from ucloud.core.client import Client
from ucloud.services.stepflow.schemas import apis


class StepFlowClient(Client):
    def __init__(
        self, config: dict, transport=None, middleware=None, logger=None
    ):
        super(StepFlowClient, self).__init__(
            config, transport, middleware, logger
        )

    def create_sf_workflow_from_template(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateSFWorkflowFromTemplate - 导入工作流定义

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Namespace** (str) - (Required) 需要创建的工作流namespace
        - **Workflow** (str) - (Required) 描述工作流定义的base64字符串
        - **WorkflowName** (str) - (Required) 需要创建的工作流名称

        **Response**

        - **Message** (str) - 返回消息
        - **Version** (int) - 创建的工作流版本号

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.CreateSFWorkflowFromTemplateRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateSFWorkflowFromTemplate", d, **kwargs)
        return apis.CreateSFWorkflowFromTemplateResponseSchema().loads(resp)

    def get_sf_workflow_template(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """GetSFWorkflowTemplate - 导出工作流定义

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **WorkflowId** (str) - (Required) 被导出工作流的Id
        - **WorkflowVersion** (int) - 被导出工作流的版本号。取值范围：WorkflowVersion >= 1；默认会获取发布版本对应的workflow；超过最大版本会返回错误

        **Response**

        - **Message** (str) - 返回消息
        - **Version** (int) - 导出工作流的版本号
        - **Workflow** (dict) - 见 **WorkflowTemplate** 模型定义
        - **WorkflowId** (str) - 导出工作流的Id

        **Response Model**

        **Param**

        - **Name** (str) - 参数名称
        - **Type** (str) - 参数类型
        - **Value** (str) - 参数值

        **ActivityTemplate**

        - **Input** (dict) - Activity的输入
        - **Name** (str) - Activity的名字
        - **Next** (str) - 下一个Activity的名字
        - **Output** (list) - Activity的输出，详见Param
        - **RetryTimes** (str) - Activity的重试次数
        - **Timeout** (str) - Activity的超时时间
        - **Type** (str) - Activity的类型

        **WorkflowTemplate**

        - **Activites** (list) - 见 **ActivityTemplate** 模型定义
        - **Input** (list) - 见 **Param** 模型定义
        - **Output** (list) - 见 **Param** 模型定义

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.GetSFWorkflowTemplateRequestSchema().dumps(d)

        resp = self.invoke("GetSFWorkflowTemplate", d, **kwargs)
        return apis.GetSFWorkflowTemplateResponseSchema().loads(resp)



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

from ucloud.core.typesystem import schema, fields
from ucloud.services.uaccount.schemas import models


""" UAccount API Schema
"""


"""
API: CreateProject

创建项目
"""


class CreateProjectRequestSchema(schema.RequestSchema):
    """CreateProject - 创建项目"""

    fields = {"ProjectName": fields.Str(required=True, dump_to="ProjectName")}


class CreateProjectResponseSchema(schema.ResponseSchema):
    """CreateProject - 创建项目"""

    fields = {"ProjectId": fields.Str(required=True, load_from="ProjectId")}


"""
API: GetProjectList

获取项目列表
"""


class GetProjectListRequestSchema(schema.RequestSchema):
    """GetProjectList - 获取项目列表"""

    fields = {"IsFinance": fields.Str(required=False, dump_to="IsFinance")}


class GetProjectListResponseSchema(schema.ResponseSchema):
    """GetProjectList - 获取项目列表"""

    fields = {
        "ProjectCount": fields.Int(required=True, load_from="ProjectCount"),
        "ProjectSet": fields.List(
            models.ProjectListInfoSchema(),
            required=True,
            load_from="ProjectSet",
        ),
    }


"""
API: GetRegion

获取用户在各数据中心的权限等信息
"""


class GetRegionRequestSchema(schema.RequestSchema):
    """GetRegion - 获取用户在各数据中心的权限等信息"""

    fields = {}


class GetRegionResponseSchema(schema.ResponseSchema):
    """GetRegion - 获取用户在各数据中心的权限等信息"""

    fields = {
        "Regions": fields.List(
            models.RegionInfoSchema(), required=False, load_from="Regions"
        )
    }


"""
API: GetUserInfo

获取用户信息
"""


class GetUserInfoRequestSchema(schema.RequestSchema):
    """GetUserInfo - 获取用户信息"""

    fields = {}


class GetUserInfoResponseSchema(schema.ResponseSchema):
    """GetUserInfo - 获取用户信息"""

    fields = {
        "DataSet": fields.List(
            models.UserInfoSchema(), required=True, load_from="DataSet"
        )
    }


"""
API: ModifyProject

修改项目
"""


class ModifyProjectRequestSchema(schema.RequestSchema):
    """ModifyProject - 修改项目"""

    fields = {
        "ProjectId": fields.Str(required=True, dump_to="ProjectId"),
        "ProjectName": fields.Str(required=True, dump_to="ProjectName"),
    }


class ModifyProjectResponseSchema(schema.ResponseSchema):
    """ModifyProject - 修改项目"""

    fields = {}


"""
API: TerminateProject

删除项目
"""


class TerminateProjectRequestSchema(schema.RequestSchema):
    """TerminateProject - 删除项目"""

    fields = {"ProjectId": fields.Str(required=False, dump_to="ProjectId")}


class TerminateProjectResponseSchema(schema.ResponseSchema):
    """TerminateProject - 删除项目"""

    fields = {}



import datetime
import time


def concat(*args):
    """cancat strings

    >>> concat(42, 'foo', 'bar')
    '42foobar'
    """
    return "".join([str(s) for s in args])


def concat_without_dot(args):
    """replace blank

    >>> concat_without_dot('42foo bar')
    '42foobar'
    """
    return "".join([str(s) for s in args.split()])


def search_value(array, origin_key, origin_value, dest_key):
    """given origin key and value,search dest_value form array by dest_key

    >>> d = [{"UHostId": "foo", "Name": "testing"}]
    >>> search_value(d, "Name", "testing", "UHostId")
    'foo'
    """
    arr = [i.get(dest_key, "") for i in array if i[origin_key] == origin_value]
    if arr:
        return arr[0]
    return ""


def timedelta(timestamp, value, typ="days"):
    """given timestamp(10bit) and calculate relative delta time

    >>> timedelta(0, 1, "days")
    86400.0

    :param timestamp: timestamp (round to second)
    :param value: float, can as positive or negative
    :param typ: days/hours
    :return: timestamp
    """
    value = int(value)
    dt = datetime.datetime.fromtimestamp(float(timestamp))
    if typ == "days":
        dt += datetime.timedelta(days=value)
    elif typ == "hours":
        dt += datetime.timedelta(hours=value)
    return time.mktime(dt.timetuple())


def get_timestamp(length=13):
    """get current timestamp string

    >>> len(str(int(get_timestamp(10))))
    10

    :param length: length of timestamp, can only between 0 and 16
    :return:
    """
    if isinstance(length, int) and 0 < length < 17:
        return int("{:.6f}".format(time.time()).replace(".", "")[:length])

    raise ValueError("timestamp length can only between 0 and 16.")



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

import typing


from ucloud.core.client import Client
from ucloud.services.udisk.schemas import apis


class UDiskClient(Client):
    def __init__(
        self, config: dict, transport=None, middleware=None, logger=None
    ):
        super(UDiskClient, self).__init__(config, transport, middleware, logger)

    def attach_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """AttachUDisk - 将一个可用的UDisk挂载到某台主机上，当UDisk挂载成功后，还需要在主机内部进行文件系统操作

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDiskId** (str) - (Required) 需要挂载的UDisk实例ID.
        - **UHostId** (str) - (Required) UHost实例ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **MultiAttach** (str) - 是否允许多点挂载（Yes: 允许多点挂载， No: 不允许多点挂载， 不填默认Yes ）

        **Response**

        - **UDiskId** (str) - 挂载的UDisk实例ID
        - **UHostId** (str) - 挂载的UHost实例ID

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.AttachUDiskRequestSchema().dumps(d)

        resp = self.invoke("AttachUDisk", d, **kwargs)
        return apis.AttachUDiskResponseSchema().loads(resp)

    def clone_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """CloneUDisk - 从UDisk创建UDisk克隆

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 实例名称
        - **SourceId** (str) - (Required) 克隆父Disk的Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year , Month, Dynamic，Postpay 默认: Dynamic
        - **Comment** (str) - Disk注释
        - **CouponId** (str) - 使用的代金券id
        - **Quantity** (int) - 购买时长 默认: 1
        - **UDataArkMode** (str) - 方舟是否开启，"Yes":开启，"No":关闭；默认为"No"

        **Response**

        - **UDiskId** (list) - 创建UDisk Id

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.CloneUDiskRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CloneUDisk", d, **kwargs)
        return apis.CloneUDiskResponseSchema().loads(resp)

    def clone_udisk_snapshot(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CloneUDiskSnapshot - 从快照创建UDisk克隆

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 实例名称
        - **Size** (int) - (Required) 购买UDisk大小,单位:GB,范围[1~2000], 权限位控制可达8T,若需要请申请开通相关权限。
        - **SourceId** (str) - (Required) 克隆父Snapshot的Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year , Month, Dynamic，Postpay 默认: Dynamic
        - **Comment** (str) - Disk注释
        - **CouponId** (str) - 使用的代金券id
        - **Quantity** (int) - 购买时长 默认: 1
        - **UDataArkMode** (str) - 是否开启数据方舟   默认:No

        **Response**

        - **UDiskId** (list) - 创建UDisk Id

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.CloneUDiskSnapshotRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CloneUDiskSnapshot", d, **kwargs)
        return apis.CloneUDiskSnapshotResponseSchema().loads(resp)

    def create_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """CreateUDisk - 创建UDisk磁盘

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 实例名称
        - **Size** (int) - (Required) 购买UDisk大小,单位:GB,普通盘: 范围[1~2000], 权限位控制可达8T,若需要请申请开通相关权限;SSD盘： 范围[1~4000]。
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year , Month, Dynamic, Postpay, Trial 默认: Dynamic
        - **CmkId** (str) - 加密需要的cmk id，UKmsMode为Yes时，必填
        - **CouponId** (str) - 使用的代金券id
        - **DiskType** (str) - UDisk 类型: DataDisk（普通数据盘），SSDDataDisk（SSD数据盘），RSSDDataDisk（RSSD数据盘），默认值（DataDisk）
        - **Quantity** (int) - 购买时长 默认: 1
        - **Tag** (str) - 业务组 默认：Default
        - **UDataArkMode** (str) - 是否开启数据方舟
        - **UKmsMode** (str) - 是否加密。Yes：加密，No：不加密，默认值（No）

        **Response**

        - **UDiskId** (list) - UDisk实例Id

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.CreateUDiskRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateUDisk", d, **kwargs)
        return apis.CreateUDiskResponseSchema().loads(resp)

    def create_udisk_snapshot(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateUDiskSnapshot - 创建snapshot快照

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Name** (str) - (Required) 快照名称
        - **UDiskId** (str) - (Required) 快照的UDisk的Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year , Month, Dynamic 默认: Dynamic
        - **Comment** (str) - 快照描述
        - **Quantity** (int) - 购买时长 默认: 1

        **Response**

        - **SnapshotId** (list) - 快照Id

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.CreateUDiskSnapshotRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateUDiskSnapshot", d, **kwargs)
        return apis.CreateUDiskSnapshotResponseSchema().loads(resp)

    def delete_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """DeleteUDisk - 删除UDisk

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDiskId** (str) - (Required) 要删除的UDisk的Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DeleteUDiskRequestSchema().dumps(d)

        resp = self.invoke("DeleteUDisk", d, **kwargs)
        return apis.DeleteUDiskResponseSchema().loads(resp)

    def delete_udisk_snapshot(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DeleteUDiskSnapshot - 删除Snapshot

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **SnapshotId** (str) - 快照Id(填写后不能填写UDisk Id)
        - **UDiskId** (str) - UDisk Id,删除该盘所创建出来的所有快照(填写后不能填写SnapshotId)

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DeleteUDiskSnapshotRequestSchema().dumps(d)

        resp = self.invoke("DeleteUDiskSnapshot", d, **kwargs)
        return apis.DeleteUDiskSnapshotResponseSchema().loads(resp)

    def describe_udisk(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUDisk - 获取UDisk实例

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **DiskType** (str) - ProtocolVersion字段为1时，需结合IsBoot确定具体磁盘类型:普通数据盘：DiskType:"CLOUD_NORMAL",IsBoot:"False"； 普通系统盘：DiskType:"CLOUD_NORMAL",IsBoot:"True"；SSD数据盘：DiskType:"CLOUD_SSD",IsBoot:"False"；SSD系统盘：DiskType:"CLOUD_SSD",IsBoot:"True"；RSSD数据盘：DiskType:"CLOUD_RSSD",IsBoot:"False"；为空拉取所有。ProtocolVersion字段为0或没有该字段时，可设为以下几个值:普通数据盘：DataDisk；普通系统盘；SystemDisk；SSD数据盘：SSDDataDisk；SSD系统盘：SSDSystemDisk；RSSD数据盘：RSSDDataDisk；为空拉取所有。
        - **IsBoot** (str) - ProtocolVersion字段为1且DiskType不为空时，必须设置，设置规则请参照DiskType；ProtocolVersion字段为1且DiskType为空时，该字段无效。ProtocolVersion字段为0或没有该字段时，该字段无效。
        - **Limit** (int) - 返回数据长度, 默认为20
        - **Offset** (int) - 数据偏移量, 默认为0
        - **ProtocolVersion** (int) - 请求协议版本，建议升级为1，为1时DiskType与UHost磁盘类型定义一致；默认为0
        - **UDiskId** (str) - UDisk Id(留空返回全部)
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **UDiskDataSet** 模型定义
        - **TotalCount** (int) - 根据过滤条件得到的总数

        **Response Model**

        **UDiskDataSet**

        - **ArkSwitchEnable** (int) - 是否支持开启方舟，1支持 ，0不支持
        - **ChargeType** (str) - Year,Month,Dynamic,Trial,Postpay
        - **CloneEnable** (int) - 是否支持克隆，1支持 ，0不支持
        - **CmkId** (str) - 该盘的cmk id
        - **CmkIdAlias** (str) - cmk id 别名
        - **CmkIdStatus** (str) - 该盘cmk的状态, Enabled(正常)，Disabled(失效)，Deleted(删除)，NoCmkId(非加密盘)
        - **CreateTime** (int) - 创建时间
        - **DataKey** (str) - 该盘的密文密钥
        - **DeviceName** (str) - 挂载的设备名称
        - **DiskType** (str) - 请求中的ProtocolVersion字段为1时，需结合IsBoot确定具体磁盘类型:普通数据盘：DiskType:"CLOUD_NORMAL",IsBoot:"False"； 普通系统盘：DiskType:"CLOUD_NORMAL",IsBoot:"True"；SSD数据盘：DiskType:"CLOUD_SSD",IsBoot:"False"；SSD系统盘：DiskType:"CLOUD_SSD",IsBoot:"True"；RSSD数据盘：DiskType:"CLOUD_RSSD",IsBoot:"False"。请求中的ProtocolVersion字段为0或没有该字段时，云硬盘类型参照如下:普通数据盘：DataDisk；普通系统盘：SystemDisk；SSD数据盘：SSDDataDisk；SSD系统盘：SSDSystemDisk；RSSD数据盘：RSSDDataDisk。
        - **ExpiredTime** (int) - 过期时间
        - **IsBoot** (str) - 是否是系统盘，是："True", 否："False"
        - **IsExpire** (str) - 资源是否过期，过期:"Yes", 未过期:"No"
        - **Name** (str) - 实例名称
        - **Size** (int) - 容量单位GB
        - **SnapEnable** (int) - 是否支持快照，1支持 ，0不支持
        - **SnapshotCount** (int) - 该盘快照个数
        - **SnapshotLimit** (int) - 该盘快照上限
        - **Status** (str) - 状态:Available(可用),Attaching(挂载中), InUse(已挂载), Detaching(卸载中), Initializating(分配中), Failed(创建失败),Cloning(克隆中),Restoring(恢复中),RestoreFailed(恢复失败),
        - **Tag** (str) - 业务组名称
        - **UDataArkMode** (str) - 是否开启数据方舟，开启:"Yes", 不支持:"No"
        - **UDiskId** (str) - UDisk实例Id
        - **UHostIP** (str) - 挂载的UHost的IP
        - **UHostId** (str) - 挂载的UHost的Id
        - **UHostName** (str) - 挂载的UHost的Name
        - **UKmsMode** (str) - 是否是加密盘，是:"Yes", 否:"No"
        - **Version** (str) - 是否支持数据方舟，支持:"2.0", 不支持:"1.0"
        - **Zone** (str) - 可用区

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DescribeUDiskRequestSchema().dumps(d)

        resp = self.invoke("DescribeUDisk", d, **kwargs)
        return apis.DescribeUDiskResponseSchema().loads(resp)

    def describe_udisk_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUDiskPrice - 获取UDisk实例价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 购买UDisk大小,单位:GB,范围[1~1000]
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - Year， Month， Dynamic，Trial，默认: Dynamic
        - **DiskType** (str) - UDisk 类型: DataDisk（普通数据盘），SSDDataDisk（SSD数据盘），SystemDisk（普通系统盘），SSDSystemDisk（SSD系统盘），RSSDDataDisk（RSSD数据盘），默认值（DataDisk）
        - **Quantity** (int) - 购买UDisk的时长，默认值为1
        - **UDataArkMode** (str) - 是否打开数据方舟, 打开"Yes",关闭"No", 默认关闭

        **Response**

        - **DataSet** (list) - 见 **UDiskPriceDataSet** 模型定义

        **Response Model**

        **UDiskPriceDataSet**

        - **ChargeName** (str) - "UDataArk","UDisk"
        - **ChargeType** (str) - Year， Month， Dynamic，Trial
        - **OriginalPrice** (int) - 用户折后价
        - **Price** (int) - 价格 (单位: 分)

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DescribeUDiskPriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUDiskPrice", d, **kwargs)
        return apis.DescribeUDiskPriceResponseSchema().loads(resp)

    def describe_udisk_snapshot(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUDiskSnapshot - 获取UDisk快照

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Limit** (int) - 返回数据长度, 默认为20
        - **Offset** (int) - 数据偏移量, 默认为0
        - **SnapshotId** (str) - 快照id，SnapshotId , UDiskId 同时传SnapshotId优先
        - **UDiskId** (str) - UDiskId,返回该盘所做快照.(必须同时传Zone)
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **DataSet** (list) - 见 **UDiskSnapshotSet** 模型定义
        - **TotalCount** (int) - 根据过滤条件得到的总数

        **Response Model**

        **UDiskSnapshotSet**

        - **CmkId** (str) - 该快照的cmk id
        - **CmkIdAlias** (str) - cmk id 别名
        - **CmkIdStatus** (str) - 该快照cmk的状态, Enabled(正常)，Disabled(失效)，Deleted(删除)，NoCmkId(非加密盘)
        - **Comment** (str) - 快照描述
        - **CreateTime** (int) - 创建时间
        - **DataKey** (str) - 该快照的密文密钥
        - **DiskType** (int) - 磁盘类型，0:数据盘，1:系统盘
        - **ExpiredTime** (int) - 过期时间
        - **IsUDiskAvailable** (bool) - 对应磁盘是否处于可用状态
        - **Name** (str) - 快照名称
        - **Size** (int) - 容量单位GB
        - **SnapshotId** (str) - 快照Id
        - **Status** (str) - 快照状态，Normal:正常,Failed:失败,Creating:制作中
        - **UDiskId** (str) - 快照的源UDisk的Id
        - **UDiskName** (str) - 快照的源UDisk的Name
        - **UHostId** (str) - 对应磁盘制作快照时所挂载的主机
        - **UKmsMode** (str) - 是否是加密盘快照，是:"Yes", 否:"No"
        - **Version** (str) - 快照版本

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DescribeUDiskSnapshotRequestSchema().dumps(d)

        resp = self.invoke("DescribeUDiskSnapshot", d, **kwargs)
        return apis.DescribeUDiskSnapshotResponseSchema().loads(resp)

    def describe_udisk_upgrade_price(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DescribeUDiskUpgradePrice - 获取UDisk升级价格信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 购买UDisk大小,单位:GB,范围[1~2000], 权限位控制可达8T,若需要请申请开通相关权限。
        - **SourceId** (str) - (Required) 升级目标UDisk ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **DiskType** (str) - 磁盘类型，SSDDataDisk:ssd数据盘,DataDisk:普通数据盘,SystemDisk:普通系统盘,SSDSystemDisk:ssd系统盘。默认为DataDisk
        - **UDataArkMode** (str) - 是否打开数据方舟, 打开"Yes",关闭"No", 默认关闭

        **Response**

        - **OriginalPrice** (int) - 用户折后价
        - **Price** (int) - 价格

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DescribeUDiskUpgradePriceRequestSchema().dumps(d)

        resp = self.invoke("DescribeUDiskUpgradePrice", d, **kwargs)
        return apis.DescribeUDiskUpgradePriceResponseSchema().loads(resp)

    def detach_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """DetachUDisk - 卸载某个已经挂载在指定UHost实例上的UDisk

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDiskId** (str) - (Required) 需要卸载的UDisk实例ID
        - **UHostId** (str) - (Required) UHost实例ID
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **UDiskId** (str) - 卸载的UDisk实例ID
        - **UHostId** (str) - 卸载的UHost实例ID

        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.DetachUDiskRequestSchema().dumps(d)

        resp = self.invoke("DetachUDisk", d, **kwargs)
        return apis.DetachUDiskResponseSchema().loads(resp)

    def rename_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """RenameUDisk - 重命名UDisk

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDiskId** (str) - (Required) 重命名的UDisk的Id
        - **UDiskName** (str) - (Required) 重命名UDisk的name
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.RenameUDiskRequestSchema().dumps(d)

        resp = self.invoke("RenameUDisk", d, **kwargs)
        return apis.RenameUDiskResponseSchema().loads(resp)

    def resize_udisk(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """ResizeUDisk - 调整UDisk容量

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Size** (int) - (Required) 调整后大小, 单位:GB, 范围[1~2000],权限位控制可达8000,若需要请申请开通相关权限。
        - **UDiskId** (str) - (Required) UDisk Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CouponId** (str) - 使用的代金券id

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.ResizeUDiskRequestSchema().dumps(d)

        resp = self.invoke("ResizeUDisk", d, **kwargs)
        return apis.ResizeUDiskResponseSchema().loads(resp)

    def restore_udisk(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """RestoreUDisk - 从备份恢复数据至UDisk

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDiskId** (str) - (Required) 需要恢复的盘id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **SnapshotId** (str) - 从指定的快照恢复
        - **SnapshotTime** (int) - 指定从方舟恢复的备份时间点

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.RestoreUDiskRequestSchema().dumps(d)

        resp = self.invoke("RestoreUDisk", d, **kwargs)
        return apis.RestoreUDiskResponseSchema().loads(resp)

    def set_udisk__udataark_mode(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """SetUDiskUDataArkMode - 设置UDisk数据方舟的状态

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **UDataArkMode** (str) - (Required) 是否开启数据方舟，开启:"Yes", 不支持:"No"
        - **UDiskId** (str) - (Required) 需要设置数据方舟的UDisk的Id
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {"ProjectId": self.config.project_id, "Region": self.config.region}
        req and d.update(req)
        d = apis.SetUDiskUDataArkModeRequestSchema().dumps(d)

        resp = self.invoke("SetUDiskUDataArkMode", d, **kwargs)
        return apis.SetUDiskUDataArkModeResponseSchema().loads(resp)



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

from ucloud.core.typesystem import schema, fields


class UHostImageSetSchema(schema.ResponseSchema):
    """UHostImageSet - DescribeImage"""

    fields = {
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "Features": fields.List(fields.Str()),
        "FuncType": fields.Str(required=False, load_from="FuncType"),
        "ImageDescription": fields.Str(
            required=False, load_from="ImageDescription"
        ),
        "ImageId": fields.Str(required=False, load_from="ImageId"),
        "ImageName": fields.Str(required=False, load_from="ImageName"),
        "ImageSize": fields.Int(required=False, load_from="ImageSize"),
        "ImageType": fields.Str(required=False, load_from="ImageType"),
        "IntegratedSoftware": fields.Str(
            required=False, load_from="IntegratedSoftware"
        ),
        "Links": fields.Str(required=False, load_from="Links"),
        "MinimalCPU": fields.Str(required=False, load_from="MinimalCPU"),
        "OsName": fields.Str(required=False, load_from="OsName"),
        "OsType": fields.Str(required=False, load_from="OsType"),
        "State": fields.Str(required=False, load_from="State"),
        "Vendor": fields.Str(required=False, load_from="Vendor"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class SpreadInfoSchema(schema.ResponseSchema):
    """SpreadInfo - 每个可用区中硬件隔离组信息"""

    fields = {
        "UHostCount": fields.Int(required=False, load_from="UHostCount"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class IsolationGroupSchema(schema.ResponseSchema):
    """IsolationGroup - 硬件隔离组信息"""

    fields = {
        "GroupId": fields.Str(required=False, load_from="GroupId"),
        "GroupName": fields.Str(required=False, load_from="GroupName"),
        "Remark": fields.Str(required=False, load_from="Remark"),
        "SpreadInfoSet": fields.List(SpreadInfoSchema()),
    }


class UHostDiskSetSchema(schema.ResponseSchema):
    """UHostDiskSet - DescribeUHostInstance"""

    fields = {
        "BackupType": fields.Str(required=False, load_from="BackupType"),
        "DiskId": fields.Str(required=False, load_from="DiskId"),
        "DiskType": fields.Str(required=True, load_from="DiskType"),
        "Drive": fields.Str(required=False, load_from="Drive"),
        "Encrypted": fields.Bool(required=False, load_from="Encrypted"),
        "IsBoot": fields.Str(required=True, load_from="IsBoot"),
        "Name": fields.Str(required=False, load_from="Name"),
        "Size": fields.Int(required=False, load_from="Size"),
        "Type": fields.Str(required=False, load_from="Type"),
    }


class UHostIPSetSchema(schema.ResponseSchema):
    """UHostIPSet - DescribeUHostInstance"""

    fields = {
        "Bandwidth": fields.Int(required=False, load_from="Bandwidth"),
        "Default": fields.Str(required=True, load_from="Default"),
        "IP": fields.Str(required=False, load_from="IP"),
        "IPId": fields.Str(required=False, load_from="IPId"),
        "Mac": fields.Str(required=True, load_from="Mac"),
        "SubnetId": fields.Str(required=False, load_from="SubnetId"),
        "Type": fields.Str(required=False, load_from="Type"),
        "VPCId": fields.Str(required=False, load_from="VPCId"),
        "Weight": fields.Int(required=True, load_from="Weight"),
    }


class UHostInstanceSetSchema(schema.ResponseSchema):
    """UHostInstanceSet - DescribeUHostInstance"""

    fields = {
        "AutoRenew": fields.Str(required=False, load_from="AutoRenew"),
        "BasicImageId": fields.Str(required=False, load_from="BasicImageId"),
        "BasicImageName": fields.Str(
            required=False, load_from="BasicImageName"
        ),
        "BootDiskState": fields.Str(required=False, load_from="BootDiskState"),
        "CPU": fields.Int(required=False, load_from="CPU"),
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "DiskSet": fields.List(UHostDiskSetSchema()),
        "ExpireTime": fields.Int(required=False, load_from="ExpireTime"),
        "GPU": fields.Int(required=False, load_from="GPU"),
        "HostType": fields.Str(required=False, load_from="HostType"),
        "HotplugFeature": fields.Bool(
            required=False, load_from="HotplugFeature"
        ),
        "IPSet": fields.List(UHostIPSetSchema()),
        "ImageId": fields.Str(required=False, load_from="ImageId"),
        "IsolationGroup": fields.Str(
            required=False, load_from="IsolationGroup"
        ),
        "LifeCycle": fields.Str(required=False, load_from="LifeCycle"),
        "MachineType": fields.Str(required=False, load_from="MachineType"),
        "Memory": fields.Int(required=False, load_from="Memory"),
        "Name": fields.Str(required=False, load_from="Name"),
        "NetCapability": fields.Str(required=False, load_from="NetCapability"),
        "NetworkState": fields.Str(required=False, load_from="NetworkState"),
        "OsName": fields.Str(required=False, load_from="OsName"),
        "OsType": fields.Str(required=False, load_from="OsType"),
        "Remark": fields.Str(required=False, load_from="Remark"),
        "State": fields.Str(required=False, load_from="State"),
        "StorageType": fields.Str(required=False, load_from="StorageType"),
        "SubnetType": fields.Str(required=False, load_from="SubnetType"),
        "Tag": fields.Str(required=False, load_from="Tag"),
        "TimemachineFeature": fields.Str(
            required=False, load_from="TimemachineFeature"
        ),
        "TotalDiskSpace": fields.Int(
            required=False, load_from="TotalDiskSpace"
        ),
        "UHostId": fields.Str(required=False, load_from="UHostId"),
        "UHostType": fields.Str(required=False, load_from="UHostType"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class UHostSnapshotSetSchema(schema.ResponseSchema):
    """UHostSnapshotSet - DescribeUHostInstanceSnapshot"""

    fields = {
        "SnapshotName": fields.Str(required=False, load_from="SnapshotName"),
        "SnapshotState": fields.Str(required=False, load_from="SnapshotState"),
        "SnapshotTime": fields.Str(required=False, load_from="SnapshotTime"),
    }


class UHostTagSetSchema(schema.ResponseSchema):
    """UHostTagSet - DescribeUHostTags"""

    fields = {
        "Tag": fields.Str(required=False, load_from="Tag"),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class UHostPriceSetSchema(schema.ResponseSchema):
    """UHostPriceSet - 主机价格"""

    fields = {
        "ChargeType": fields.Str(required=True, load_from="ChargeType"),
        "Price": fields.Float(required=True, load_from="Price"),
    }



""" Code is generated by ucloud-model, DO NOT EDIT IT. """


from ucloud.core.typesystem import schema, fields
from ucloud.services.umem.schemas import models

""" UMem API Schema
"""


"""
API: CreateUMemBackup

创建分布式redis备份
"""


class CreateUMemBackupRequestSchema(schema.RequestSchema):
    """CreateUMemBackup - 创建分布式redis备份"""

    fields = {
        "BackupName": fields.Str(required=True, dump_to="BackupName"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class CreateUMemBackupResponseSchema(schema.ResponseSchema):
    """CreateUMemBackup - 创建分布式redis备份"""

    fields = {
        "BackupId": fields.Str(required=False, load_from="BackupId"),
    }


"""
API: CreateUMemSpace

创建UMem内存空间
"""


class CreateUMemSpaceRequestSchema(schema.RequestSchema):
    """CreateUMemSpace - 创建UMem内存空间"""

    fields = {
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "CouponId": fields.Str(required=False, dump_to="CouponId"),
        "Name": fields.Str(required=True, dump_to="Name"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Protocol": fields.Str(required=False, dump_to="Protocol"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "SubnetId": fields.Str(required=False, dump_to="SubnetId"),
        "Type": fields.Str(required=False, dump_to="Type"),
        "VPCId": fields.Str(required=False, dump_to="VPCId"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class CreateUMemSpaceResponseSchema(schema.ResponseSchema):
    """CreateUMemSpace - 创建UMem内存空间"""

    fields = {
        "SpaceId": fields.Str(required=False, load_from="SpaceId"),
    }


"""
API: CreateUMemcacheGroup

创建单机Memcache
"""


class CreateUMemcacheGroupRequestSchema(schema.RequestSchema):
    """CreateUMemcacheGroup - 创建单机Memcache"""

    fields = {
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "ConfigId": fields.Str(required=False, dump_to="ConfigId"),
        "CouponId": fields.Str(required=False, dump_to="CouponId"),
        "Name": fields.Str(required=True, dump_to="Name"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Protocol": fields.Str(required=False, dump_to="Protocol"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=False, dump_to="Size"),
        "SubnetId": fields.Str(required=False, dump_to="SubnetId"),
        "Tag": fields.Str(required=False, dump_to="Tag"),
        "VPCId": fields.Str(required=False, dump_to="VPCId"),
        "Version": fields.Str(required=False, dump_to="Version"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class CreateUMemcacheGroupResponseSchema(schema.ResponseSchema):
    """CreateUMemcacheGroup - 创建单机Memcache"""

    fields = {
        "GroupId": fields.Str(required=False, load_from="GroupId"),
    }


"""
API: CreateURedisBackup

创建主备Redis备份
"""


class CreateURedisBackupRequestSchema(schema.RequestSchema):
    """CreateURedisBackup - 创建主备Redis备份"""

    fields = {
        "BackupName": fields.Str(required=True, dump_to="BackupName"),
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SlaveZone": fields.Str(required=False, dump_to="SlaveZone"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class CreateURedisBackupResponseSchema(schema.ResponseSchema):
    """CreateURedisBackup - 创建主备Redis备份"""

    fields = {
        "BackupId": fields.Str(required=True, load_from="BackupId"),
    }


"""
API: CreateURedisGroup

创建主备redis
"""


class CreateURedisGroupRequestSchema(schema.RequestSchema):
    """CreateURedisGroup - 创建主备redis"""

    fields = {
        "AutoBackup": fields.Str(required=False, dump_to="AutoBackup"),
        "BackupId": fields.Str(required=False, dump_to="BackupId"),
        "BackupTime": fields.Int(required=False, dump_to="BackupTime"),
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "ConfigId": fields.Str(required=False, dump_to="ConfigId"),
        "CouponId": fields.Str(required=False, dump_to="CouponId"),
        "EnableIpV6": fields.Bool(required=False, dump_to="EnableIpV6"),
        "HighAvailability": fields.Str(
            required=True, dump_to="HighAvailability"
        ),
        "MasterGroupId": fields.Str(required=False, dump_to="MasterGroupId"),
        "Name": fields.Str(required=True, dump_to="Name"),
        "Password": fields.Base64(required=False, dump_to="Password"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=False, dump_to="Size"),
        "SlaveZone": fields.Str(required=False, dump_to="SlaveZone"),
        "SubnetId": fields.Str(required=False, dump_to="SubnetId"),
        "Tag": fields.Str(required=False, dump_to="Tag"),
        "VPCId": fields.Str(required=False, dump_to="VPCId"),
        "Version": fields.Str(required=False, dump_to="Version"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class CreateURedisGroupResponseSchema(schema.ResponseSchema):
    """CreateURedisGroup - 创建主备redis"""

    fields = {
        "GroupId": fields.Str(required=False, load_from="GroupId"),
    }


"""
API: DeleteUMemSpace

删除UMem内存空间
"""


class DeleteUMemSpaceRequestSchema(schema.RequestSchema):
    """DeleteUMemSpace - 删除UMem内存空间"""

    fields = {
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DeleteUMemSpaceResponseSchema(schema.ResponseSchema):
    """DeleteUMemSpace - 删除UMem内存空间"""

    fields = {}


"""
API: DeleteUMemcacheGroup

删除单机Memcache
"""


class DeleteUMemcacheGroupRequestSchema(schema.RequestSchema):
    """DeleteUMemcacheGroup - 删除单机Memcache"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DeleteUMemcacheGroupResponseSchema(schema.ResponseSchema):
    """DeleteUMemcacheGroup - 删除单机Memcache"""

    fields = {}


"""
API: DeleteURedisGroup

删除主备redis
"""


class DeleteURedisGroupRequestSchema(schema.RequestSchema):
    """DeleteURedisGroup - 删除主备redis"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
    }


class DeleteURedisGroupResponseSchema(schema.ResponseSchema):
    """DeleteURedisGroup - 删除主备redis"""

    fields = {}


"""
API: DescribeUMemBackup

查询分布式redis备份
"""


class DescribeUMemBackupRequestSchema(schema.RequestSchema):
    """DescribeUMemBackup - 查询分布式redis备份"""

    fields = {
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeUMemBackupResponseSchema(schema.ResponseSchema):
    """DescribeUMemBackup - 查询分布式redis备份"""

    fields = {
        "DataSet": fields.List(
            models.UMemBackupSetSchema(), required=False, load_from="DataSet"
        ),
    }


"""
API: DescribeUMemBackupURL

获取分布式redis 备份下载链接
"""


class DescribeUMemBackupURLRequestSchema(schema.RequestSchema):
    """DescribeUMemBackupURL - 获取分布式redis 备份下载链接"""

    fields = {
        "BackupId": fields.Str(required=True, dump_to="BackupId"),
        "BlockId": fields.Str(required=False, dump_to="BlockId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeUMemBackupURLResponseSchema(schema.ResponseSchema):
    """DescribeUMemBackupURL - 获取分布式redis 备份下载链接"""

    fields = {
        "BackupURL": fields.List(
            fields.Str(), required=True, load_from="BackupURL"
        ),
    }


"""
API: DescribeUMemBlockInfo

拉取UDRedis分片信息
"""


class DescribeUMemBlockInfoRequestSchema(schema.RequestSchema):
    """DescribeUMemBlockInfo - 拉取UDRedis分片信息"""

    fields = {
        "Limit": fields.Int(required=True, dump_to="Limit"),
        "Offset": fields.Int(required=True, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeUMemBlockInfoResponseSchema(schema.ResponseSchema):
    """DescribeUMemBlockInfo - 拉取UDRedis分片信息"""

    fields = {
        "DataSet": fields.List(
            models.UMemBlockInfoSchema(), required=False, load_from="DataSet"
        ),
    }


"""
API: DescribeUMemPrice

获取UMem实例价格信息
"""


class DescribeUMemPriceRequestSchema(schema.RequestSchema):
    """DescribeUMemPrice - 获取UMem实例价格信息"""

    fields = {
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "ProjectId": fields.Str(required=True, dump_to="ProjectId"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "Type": fields.Str(required=True, dump_to="Type"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeUMemPriceResponseSchema(schema.ResponseSchema):
    """DescribeUMemPrice - 获取UMem实例价格信息"""

    fields = {
        "DataSet": fields.List(
            models.UMemPriceSetSchema(), required=False, load_from="DataSet"
        ),
    }


"""
API: DescribeUMemSpace

获取UMem内存空间列表
"""


class DescribeUMemSpaceRequestSchema(schema.RequestSchema):
    """DescribeUMemSpace - 获取UMem内存空间列表"""

    fields = {
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Protocol": fields.Str(required=False, dump_to="Protocol"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=False, dump_to="SpaceId"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeUMemSpaceResponseSchema(schema.ResponseSchema):
    """DescribeUMemSpace - 获取UMem内存空间列表"""

    fields = {
        "DataSet": fields.List(
            models.UMemSpaceSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: DescribeUMemUpgradePrice

获取UMem升级价格信息
"""


class DescribeUMemUpgradePriceRequestSchema(schema.RequestSchema):
    """DescribeUMemUpgradePrice - 获取UMem升级价格信息"""

    fields = {
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Type": fields.Str(required=True, dump_to="Type"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeUMemUpgradePriceResponseSchema(schema.ResponseSchema):
    """DescribeUMemUpgradePrice - 获取UMem升级价格信息"""

    fields = {
        "DataSet": models.PriceDataSetSchema(),
        "Price": fields.Int(required=False, load_from="Price"),
    }


"""
API: DescribeUMemcacheGroup

显示Memcache
"""


class DescribeUMemcacheGroupRequestSchema(schema.RequestSchema):
    """DescribeUMemcacheGroup - 显示Memcache"""

    fields = {
        "GroupId": fields.Str(required=False, dump_to="GroupId"),
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeUMemcacheGroupResponseSchema(schema.ResponseSchema):
    """DescribeUMemcacheGroup - 显示Memcache"""

    fields = {
        "DataSet": fields.List(
            models.UMemcacheGroupSetSchema(),
            required=False,
            load_from="DataSet",
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: DescribeUMemcachePrice

获取umemcache组价格信息
"""


class DescribeUMemcachePriceRequestSchema(schema.RequestSchema):
    """DescribeUMemcachePrice - 获取umemcache组价格信息"""

    fields = {
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "Type": fields.Str(required=False, dump_to="Type"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeUMemcachePriceResponseSchema(schema.ResponseSchema):
    """DescribeUMemcachePrice - 获取umemcache组价格信息"""

    fields = {
        "DataSet": fields.List(
            models.UMemcachePriceSetSchema(),
            required=False,
            load_from="DataSet",
        ),
    }


"""
API: DescribeUMemcacheUpgradePrice

获取umemcache升级价格信息
"""


class DescribeUMemcacheUpgradePriceRequestSchema(schema.RequestSchema):
    """DescribeUMemcacheUpgradePrice - 获取umemcache升级价格信息"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "Size": fields.Int(required=True, dump_to="Size"),
    }


class DescribeUMemcacheUpgradePriceResponseSchema(schema.ResponseSchema):
    """DescribeUMemcacheUpgradePrice - 获取umemcache升级价格信息"""

    fields = {
        "Price": fields.Float(required=False, load_from="Price"),
    }


"""
API: DescribeURedisBackup

查询主备redis备份
"""


class DescribeURedisBackupRequestSchema(schema.RequestSchema):
    """DescribeURedisBackup - 查询主备redis备份"""

    fields = {
        "GroupId": fields.Str(required=False, dump_to="GroupId"),
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
    }


class DescribeURedisBackupResponseSchema(schema.ResponseSchema):
    """DescribeURedisBackup - 查询主备redis备份"""

    fields = {
        "DataSet": fields.List(
            models.URedisBackupSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: DescribeURedisBackupURL

获取主备Redis备份下载链接
"""


class DescribeURedisBackupURLRequestSchema(schema.RequestSchema):
    """DescribeURedisBackupURL - 获取主备Redis备份下载链接"""

    fields = {
        "BackupId": fields.Str(required=True, dump_to="BackupId"),
        "GroupId": fields.Str(required=False, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "RegionFlag": fields.Bool(required=False, dump_to="RegionFlag"),
        "SlaveZone": fields.Str(required=False, dump_to="SlaveZone"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeURedisBackupURLResponseSchema(schema.ResponseSchema):
    """DescribeURedisBackupURL - 获取主备Redis备份下载链接"""

    fields = {
        "BackupPath": fields.Str(required=False, load_from="BackupPath"),
        "BackupURL": fields.Str(required=False, load_from="BackupURL"),
    }


"""
API: DescribeURedisConfig

查询主备Redis所有配置文件
"""


class DescribeURedisConfigRequestSchema(schema.RequestSchema):
    """DescribeURedisConfig - 查询主备Redis所有配置文件"""

    fields = {
        "ConfigId": fields.Str(required=False, dump_to="ConfigId"),
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "RegionFlag": fields.Bool(required=True, dump_to="RegionFlag"),
        "Version": fields.Str(required=False, dump_to="Version"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeURedisConfigResponseSchema(schema.ResponseSchema):
    """DescribeURedisConfig - 查询主备Redis所有配置文件"""

    fields = {
        "DataSet": fields.List(
            models.URedisConfigSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: DescribeURedisGroup

查询主备Redis
"""


class DescribeURedisGroupRequestSchema(schema.RequestSchema):
    """DescribeURedisGroup - 查询主备Redis"""

    fields = {
        "GroupId": fields.Str(required=False, dump_to="GroupId"),
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "Offset": fields.Int(required=False, dump_to="Offset"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeURedisGroupResponseSchema(schema.ResponseSchema):
    """DescribeURedisGroup - 查询主备Redis"""

    fields = {
        "DataSet": fields.List(
            models.URedisGroupSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: DescribeURedisPrice

取uredis价格信息
"""


class DescribeURedisPriceRequestSchema(schema.RequestSchema):
    """DescribeURedisPrice - 取uredis价格信息"""

    fields = {
        "ChargeType": fields.Str(required=False, dump_to="ChargeType"),
        "ProductType": fields.Str(required=False, dump_to="ProductType"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Quantity": fields.Int(required=False, dump_to="Quantity"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "RegionFlag": fields.Bool(required=False, dump_to="RegionFlag"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeURedisPriceResponseSchema(schema.ResponseSchema):
    """DescribeURedisPrice - 取uredis价格信息"""

    fields = {
        "DataSet": fields.List(
            models.URedisPriceSetSchema(), required=False, load_from="DataSet"
        ),
    }


"""
API: DescribeURedisSlowlog

查询URedis慢日志
"""


class DescribeURedisSlowlogRequestSchema(schema.RequestSchema):
    """DescribeURedisSlowlog - 查询URedis慢日志"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "Limit": fields.Int(required=False, dump_to="Limit"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeURedisSlowlogResponseSchema(schema.ResponseSchema):
    """DescribeURedisSlowlog - 查询URedis慢日志"""

    fields = {
        "DataSet": fields.List(
            models.URedisSlowlogSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=True, load_from="TotalCount"),
    }


"""
API: DescribeURedisUpgradePrice

获取uredis升级价格信息
"""


class DescribeURedisUpgradePriceRequestSchema(schema.RequestSchema):
    """DescribeURedisUpgradePrice - 获取uredis升级价格信息"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class DescribeURedisUpgradePriceResponseSchema(schema.ResponseSchema):
    """DescribeURedisUpgradePrice - 获取uredis升级价格信息"""

    fields = {
        "Price": fields.Float(required=False, load_from="Price"),
    }


"""
API: DescribeURedisVersion

获取主Redis可用版本
"""


class DescribeURedisVersionRequestSchema(schema.RequestSchema):
    """DescribeURedisVersion - 获取主Redis可用版本"""

    fields = {
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class DescribeURedisVersionResponseSchema(schema.ResponseSchema):
    """DescribeURedisVersion - 获取主Redis可用版本"""

    fields = {
        "DataSet": fields.List(
            models.URedisVersionSetSchema(), required=False, load_from="DataSet"
        ),
        "TotalCount": fields.Int(required=False, load_from="TotalCount"),
    }


"""
API: GetUMemSpaceState

获取UMem内存空间列表
"""


class GetUMemSpaceStateRequestSchema(schema.RequestSchema):
    """GetUMemSpaceState - 获取UMem内存空间列表"""

    fields = {
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class GetUMemSpaceStateResponseSchema(schema.ResponseSchema):
    """GetUMemSpaceState - 获取UMem内存空间列表"""

    fields = {
        "State": fields.Str(required=False, load_from="State"),
    }


"""
API: ModifyUMemSpaceName

修改UMem内存空间名称
"""


class ModifyUMemSpaceNameRequestSchema(schema.RequestSchema):
    """ModifyUMemSpaceName - 修改UMem内存空间名称"""

    fields = {
        "Name": fields.Str(required=True, dump_to="Name"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class ModifyUMemSpaceNameResponseSchema(schema.ResponseSchema):
    """ModifyUMemSpaceName - 修改UMem内存空间名称"""

    fields = {}


"""
API: ModifyURedisGroupName

修改主备redis名称
"""


class ModifyURedisGroupNameRequestSchema(schema.RequestSchema):
    """ModifyURedisGroupName - 修改主备redis名称"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "Name": fields.Str(required=True, dump_to="Name"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
    }


class ModifyURedisGroupNameResponseSchema(schema.ResponseSchema):
    """ModifyURedisGroupName - 修改主备redis名称"""

    fields = {}


"""
API: ModifyURedisGroupPassword

修改主备密码/重置密码
"""


class ModifyURedisGroupPasswordRequestSchema(schema.RequestSchema):
    """ModifyURedisGroupPassword - 修改主备密码/重置密码"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "Password": fields.Str(required=True, dump_to="Password"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "ResourceType": fields.Str(required=False, dump_to="ResourceType"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class ModifyURedisGroupPasswordResponseSchema(schema.ResponseSchema):
    """ModifyURedisGroupPassword - 修改主备密码/重置密码"""

    fields = {}


"""
API: ResizeUMemSpace

调整内存空间容量
"""


class ResizeUMemSpaceRequestSchema(schema.RequestSchema):
    """ResizeUMemSpace - 调整内存空间容量"""

    fields = {
        "CouponId": fields.Str(required=False, dump_to="CouponId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
        "SpaceId": fields.Str(required=True, dump_to="SpaceId"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class ResizeUMemSpaceResponseSchema(schema.ResponseSchema):
    """ResizeUMemSpace - 调整内存空间容量"""

    fields = {}


"""
API: ResizeURedisGroup

调整主备redis容量
"""


class ResizeURedisGroupRequestSchema(schema.RequestSchema):
    """ResizeURedisGroup - 调整主备redis容量"""

    fields = {
        "CouponId": fields.Int(required=False, dump_to="CouponId"),
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Size": fields.Int(required=True, dump_to="Size"),
    }


class ResizeURedisGroupResponseSchema(schema.ResponseSchema):
    """ResizeURedisGroup - 调整主备redis容量"""

    fields = {}


"""
API: RestartUMemcacheGroup

重启单机Memcache
"""


class RestartUMemcacheGroupRequestSchema(schema.RequestSchema):
    """RestartUMemcacheGroup - 重启单机Memcache"""

    fields = {
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "Zone": fields.Str(required=True, dump_to="Zone"),
    }


class RestartUMemcacheGroupResponseSchema(schema.ResponseSchema):
    """RestartUMemcacheGroup - 重启单机Memcache"""

    fields = {}


"""
API: UpdateURedisBackupStrategy

URedisBackupStrategy
"""


class UpdateURedisBackupStrategyRequestSchema(schema.RequestSchema):
    """UpdateURedisBackupStrategy - URedisBackupStrategy"""

    fields = {
        "AutoBackup": fields.Str(required=False, dump_to="AutoBackup"),
        "BackupTime": fields.Str(required=True, dump_to="BackupTime"),
        "GroupId": fields.Str(required=True, dump_to="GroupId"),
        "ProjectId": fields.Str(required=False, dump_to="ProjectId"),
        "Region": fields.Str(required=True, dump_to="Region"),
        "SlaveZone": fields.Str(required=False, dump_to="SlaveZone"),
        "Zone": fields.Str(required=False, dump_to="Zone"),
    }


class UpdateURedisBackupStrategyResponseSchema(schema.ResponseSchema):
    """UpdateURedisBackupStrategy - URedisBackupStrategy"""

    fields = {}
